@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 module Distribution.Simple.Program.GHC (
     GhcOptions(..),
     GhcMode(..),
@@ -26,7 +27,9 @@ import Distribution.Utils.NubList   ( NubListR, fromNubListR )
 import Language.Haskell.Extension   ( Language(..), Extension(..) )
 
 import qualified Data.Map as M
+#if __GLASGOW_HASKELL__ < 710
 import Data.Monoid
+#endif
 import Data.List ( intercalate )
 
 -- | A structured set of GHC options/flags
