@@ -5,12 +5,18 @@ module Main
 import System.IO (BufferMode(NoBuffering), hSetBuffering, stdout)
 import Test.Framework
 
+import qualified UnitTests.Distribution.Compat.CreatePipe
 import qualified UnitTests.Distribution.Compat.ReadP
+import qualified UnitTests.Distribution.Utils.NubList
 
 tests :: [Test]
-tests = [
-    testGroup "Distribution.Compat.ReadP"
+tests =
+    [ testGroup "Distribution.Compat.ReadP"
         UnitTests.Distribution.Compat.ReadP.tests
+    , testGroup "Distribution.Compat.CreatePipe"
+        UnitTests.Distribution.Compat.CreatePipe.tests
+    , testGroup "Distribution.Utils.NubList"
+        UnitTests.Distribution.Utils.NubList.tests
     ]
 
 main :: IO ()
